import sys, getopt
import csv


def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False


#gets the type of each column in a file with header
def get_type_from_header(input_file, MAX_ENUMERATED=3):
    fileIn = open(input_file)
    #get names of attributes
    attrs = fileIn.readline().strip().split(',')
    nattrs = len(attrs)
    atype = [True] * nattrs

    distinctValues = []
    for k in range(nattrs):
        distinctValues.append(set())

    #get types analyzing the others lines
    lines = csv.reader(fileIn, delimiter=',')
    for row in lines:
        #for each column
        for k in range(nattrs):
            if row[k] == '?':
                pass
            else:
                atype[k] = atype[k] & is_number(row[k])
                if len(distinctValues[k]) <= MAX_ENUMERATED:
                    distinctValues[k].add(row[k])
    fileIn.close()

    finalTypes = []
    for k in range(nattrs):
        if len(distinctValues[k]) > MAX_ENUMERATED:
            if atype[k] == True:
                finalTypes.append('numeric')
            else:
                finalTypes.append('string')
        else:
            #enumerated - construct string
            res = "{"
            first = True
            for s in distinctValues[k]:
                if first:
                    first = False
                else:
                    res += ","
                res += s
            res += "}"
            finalTypes.append(res)
    return attrs, finalTypes


#gets the type of each column in a file without header
def get_type_without_header(input_file, MAX_ENUMERATED=3):
    fileIn = open(input_file)

    lines = csv.reader(fileIn, delimiter=',')

    distinctValues = []
    atype = []
    nattrs = 0
    first_line = True

    for row in lines:
        if first_line:
            #Initialize data structures 
            nattrs = len(row)
            atype = [True] * nattrs
            for k in range(nattrs):
                distinctValues.append(set())
            first_line = False

        for k in range(nattrs):
            if row[k] == '?':
                pass
            else:
                atype[k] = atype[k] & is_number(row[k])
                if len(distinctValues[k]) <= MAX_ENUMERATED:
                    distinctValues[k].add(row[k])
    fileIn.close()

    finalTypes = []
    for k in range(nattrs):
        if len(distinctValues[k]) > MAX_ENUMERATED:
            if atype[k] == True:
                finalTypes.append('numeric')
            else:
                finalTypes.append('string')
        else:
            res = "{"
            first = True
            for s in distinctValues[k]:
                if first:
                    first = False
                else:
                    res += ","
                res += s
            res += "}"
            finalTypes.append(res)
    return finalTypes


def csv2arff(input_file, output_file, max_enum):
    fileOut = open(output_file, "w")
    fileOut.write("@RELATION " + output_file + "\n")
    #get types of attributes
    attTypes = get_type_without_header(input_file, MAX_ENUMERATED=max_enum)
    #write metadata
    for k in range(len(attTypes)):
        fileOut.write("@ATTRIBUTE column-" + str(k) + " " + attTypes[k] + "\n")
    #write data
    fileOut.write("@DATA \n")
    fileIn = open(input_file)
    fileOut.writelines(fileIn.readlines())
    fileIn.close()
    fileOut.close()


def csv2arff_header(input_file, output_file, max_enum):
    fileOut = open(output_file, "w")
    fileOut.write("@RELATION " + output_file + "\n")

    #get types of attributes and their names
    attrs, attTypes = get_type_from_header(input_file, MAX_ENUMERATED=max_enum)
    #write metadata
    for k in range(len(attTypes)):
        fileOut.write("@ATTRIBUTE " + attrs[k] + " " + attTypes[k] + "\n")

    #write data    
    fileOut.write("@DATA \n")
    fileIn = open(input_file)
    fileIn.readline()
    fileOut.writelines(fileIn.readlines())
    fileIn.close()
    fileOut.close()


def main():
    input_file = ''
    output_file = ''
    max_enum = None
    head = None

    # Read command line args
    my_opts, args = getopt.getopt(sys.argv[1:], "i:o:h:m:")
    ###############################
    # option is the name of the option
    # value is the actual value passed to 'option'
    ###############################
    for option, value in my_opts:
        if option == '-i':
            input_file = value
        elif option == '-o':
            output_file = value
        elif option == '-h':
            head = value
        elif option == '-m':
            max_enum = value
        else:
            raise Exception("Usage: %s -i input -o output -h y/n" % sys.argv[0])

    if head == 'y':
        csv2arff_header(input_file, output_file, max_enum)
    else:
        csv2arff(input_file, output_file, max_enum)


if __name__ == '__main__':
    main()
